(function(){
	var app = angular.module("hafe-card");
	
	app.controller("CardController", ['$scope', '$http', 'AuthService', 'PropertiesService', 'CardPreferencesService', function($scope, $http, AuthService, PropertiesService, CardPreferencesService){
		var ao = this;
		
		ao.cardMode = 'front';
		$scope.user = {};
		$scope.preferences = {};
		ao.indicatorTimeout;
		
		ao.init = function(){
			ao.loadUserInfo();
			ao.loadPreferences();
		};
		
		ao.loadUserInfo = function(){
			AuthService.getUserInfo()
			.done(function(resp){
				resp = JSON.parse(resp);
				
				if(resp.status == 'success'){
					$scope.user = resp.data.user;
					$scope.$apply();
				}
			});
		};
		
		ao.setMainColor = function(mainColor){
			$scope.preferences.mainColor = mainColor;
			$scope.$apply();
		};
		
		ao.loadProperties = function(){
			PropertiesService.getProperties()
			.done(function(resp){
				resp = JSON.parse(resp)
				
				if(resp.status == 'success'){
					$scope.preferences.properties = resp.data.properties;
					ao.sortProperties();
				}else{
					showNotification('error', false, "No hem pogut carregar les dades correctament.");
				}
			});
		};
		
		ao.sortProperties = function(reorder){
			if(reorder === undefined || reorder){
				$scope.preferences.properties.sort(function(a, b){
					var p = a.cardPosition - b.cardPosition;
					
					if(p !== 0){
						if(a.cardPosition == 0){
							return 1;
						}else if(b.cardPosition == 0){
							return -1;
						}else{
							return p;
						}
					}
					
					return a.name - b.name;
				});
			}
			
			// Resetegem l'ordre per no acabar amb números altíssims
			var selected_i = 0;
			$.each($scope.preferences.properties, function(i, p){
				if(p.cardPosition != 0){
					selected_i++;
					p.cardPosition = selected_i;
				}
			})
			
			setTimeout(function(){ 
				$scope.$apply();
			});
		};
		
		ao.switchSelection = function(prop){
			if(prop.cardPosition == 0){
				prop.cardPosition = $scope.preferences.properties.length+1;
			}else{
				prop.cardPosition = 0;
			}
			
			ao.saveCardPreferences();
			ao.sortProperties();
			ao.activateProperty(prop.id);
		};
		
		ao.loadPreferences = function(){
			CardPreferencesService.getPreferences()
			.done(function(resp){
				resp = JSON.parse(resp)
				
				if(resp.status == 'success'){
					$scope.preferences = resp.data.preferences;
					ao.loadProperties();
					$("#pref_main_color .color_selected").attr("data-selected", $scope.preferences.mainColor);
					
					$scope.$apply();
				}else{
					showNotification('error', false, "No hem pogut carregar les dades correctament.");
				}
			});
		};
		
		ao.loadQrCode = function(){
			var basic_link_prefix = global_url + "/login/";
			var emergency_link_prefix = global_url + "/send-alert/";
			
			var qrcolor = $scope.preferences.qrInverted ? 'white' : 'black';
			
			$("#basic_qr").html("");
			$("#emergency_qr").html("");
			
			var basic_qrcode = new QRCode("basic_qr", {
				text: basic_link_prefix + $scope.preferences.linkCode,
				width: $("#basic_qr").width(),
				height: $("#basic_qr").height(),
				colorDark : qrcolor,
				colorLight : "transparent",
				correctLevel : QRCode.CorrectLevel.L
			});
			
			var emergency_qrcode = new QRCode("emergency_qr", {
				text: emergency_link_prefix + $scope.preferences.linkCode,
				width: $("#emergency_qr").width(),
				height: $("#emergency_qr").height(),
				colorDark : qrcolor,
				colorLight : "transparent",
				correctLevel : QRCode.CorrectLevel.L
			});
		};
		
		ao.sortUp = function(prop){
			var moving = $scope.preferences.properties.indexOf(prop);
			
			if(moving > 0){
				if($scope.preferences.properties[moving].cardPosition > 1){
					$scope.preferences.properties[moving].cardPosition -= 1; 
					$scope.preferences.properties[moving-1].cardPosition += 1;
				}
			}
			
			ao.sortProperties();
			ao.saveCardPreferences();
			ao.activateProperty(prop.id);
		};
		
		ao.sortDown = function(prop){
			var moving = $scope.preferences.properties.indexOf(prop);
			
			if($scope.preferences.properties[moving+1].cardPosition != 0){
				$scope.preferences.properties[moving].cardPosition += 1; 
				$scope.preferences.properties[moving+1].cardPosition -= 1;
			}
			
			ao.sortProperties();
			ao.saveCardPreferences();
			ao.activateProperty(prop.id);
		};
		
		ao.saveCardPreferences = function(){
			ao.sortProperties(false);
			
			CardPreferencesService.savePreferences($scope.preferences)
			.done(function(resp){
				resp = JSON.parse(resp)
				
				if(resp.status == 'success'){
					
				}else{
					showNotification('error', false, "No s'han pogut guardar els canvis.");
				}
			});
			
			// Ho tanquem abans de rebre resposta, i així l'usuari té sensació d'immediatesa
			$('#card_preferences_edition_btn').sideNav('hide');
		};
		
		ao.swapCardFace = function(){
			if(ao.cardMode == 'front'){
				ao.cardMode = 'back';
			}else{
				ao.cardMode = 'front';
			}
			
			setTimeout(function(){ 
				$scope.$apply();
				ao.loadQrCode();
			});
			
			return ao.cardMode;
		};
		
		ao.activateProperty = function(propid){
			$(".card_properties_container .card_property").removeClass("activated");
			$('.card_property[data-id="'+propid+'"]').addClass("activated");
			
			if(ao.indicatorTimeout !== undefined){
				clearTimeout(ao.indicatorTimeout);
				ao.indicatorTimeout = undefined;
			}
			
			ao.indicatorTimeout = setTimeout(function(){
				$('.card_property[data-id="'+propid+'"]').removeClass("activated");
			}, 2500);
		};
		
		$scope.$watch('preferences.qrInverted', function() {
			ao.loadQrCode();
		});
		
		// Executem les funcions en quant es carregui el controlador
		ao.init();
		
	}]);
})();