<?php

	function _GetCardPreferences($user_id){
		
		global $db;
		
		$return_data = initialize_return_data();
		
		$get_preferences_q = "SELECT 
				cpr.id AS id,
				cpr.main_color AS main_color,
				cpr.name_highlight AS name_highlight,
				cpr.qr_inverted AS qr_inverted,
				cpr.print_centered AS print_centered,
				cpr.bleed_marks AS bleed_marks,
				us.link_code AS link_code 
			FROM card_preferences cpr 
			INNER JOIN users us
				ON us.id = cpr.user_id 
			WHERE cpr.user_id = ".aqinnesc($user_id)." 
			GROUP BY cpr.user_id";
				
		$get_preferences = $db->query($get_preferences_q);
		
		if($db->error){
			
			$return_data['status'] = 'failed';
			$return_data['error'] = '2293493728';
			$return_data['line'] = __LINE__;
			$return_data['technical']['e'] = $db->error;
			
			return json_encode($return_data);
			
		}else{
			
			if($get_preferences->num_rows == 0){
				
				$add_default = json_decode(_AddDefaultCardPreferences($user_id));
				
				if($add_default->status != 'success'){
					
					return json_encode($add_default);
					
				}
				
				return _GetCardPreferences($user_id);
				
			}else{
				
				$return_data['status'] = 'success';
				$return_data['data']['preferences'] = array();
				
				while($pref = $get_preferences->fetch_object()){
					
					$return_data['data']['preferences']['mainColor'] = $pref->main_color;
					$return_data['data']['preferences']['nameHighlight'] = $pref->name_highlight == '1';
					$return_data['data']['preferences']['qrInverted'] = $pref->qr_inverted == '1';
					$return_data['data']['preferences']['printCentered'] = $pref->print_centered == '1';
					$return_data['data']['preferences']['bleedMarks'] = $pref->bleed_marks == '1';
					$return_data['data']['preferences']['linkCode'] = $pref->link_code;
					
				}
				
			}
			
		}
		
		return json_encode($return_data);
		
	}
	
	function _SaveCardPreferences($user_id, $preferences){
		
		global $db;
		
		$return_data = initialize_return_data();
		$now = get_now();
		
		$to_still = array();
		
		// Aconseguim la ID de les preferències
		$get_cpr_q = "SELECT cpr.id AS cpr_id
			FROM card_preferences cpr 
			WHERE cpr.user_id = ".aqinnesc($user_id)." 
			GROUP BY cpr.user_id";
			
		$get_cpr = $db->query($get_cpr_q);
		
		if($get_cpr->num_rows == 0){
			
			$add_default = json_decode(_AddDefaultCardPreferences($user_id));
			$cpr_id = $add_default->data->preferences->id;
			
		}else{
		
			$cpr = $get_cpr->fetch_object();
			$cpr_id = $cpr->cpr_id;
		
		}
		
		// Inserim (o actualitzem) els camps / propietats
		$fields_stmt = $db->prepare("INSERT INTO card_fields 
				(card_pref_id, field_id, position, date_added) 
			SELECT ?, ?, ?, ? 
			FROM card_preferences cpr 
			WHERE cpr.user_id = ? 
				ON DUPLICATE KEY UPDATE position = ?");
		
		$fields_stmt->bind_param("iiisii", $cpr_id, $field_id, $field_position, $now, $user_id, $field_position);
		
		foreach($preferences->properties as $property){
			
			if(property_is_defined($property->cardPosition) && intval($property->cardPosition) > 0){
				
				$field_id = $property->id;
				$field_position = $property->cardPosition;
				
				$fields_stmt->execute();
				
				if($db->error){
					
					$return_data['status'] = 'failed';
					$return_data['error'] = '87492892937';
					$return_data['line'] = __LINE__;
					$return_data['technical']['e'] = $db->error;
					
					return json_encode($return_data);
					
				}
				
				$to_still[] = $property->id;
			
			}
			
		}
		
		// Eliminem camps antics que ja no estan seleccionats
		$delete_old_q = "DELETE FROM card_fields 
			WHERE card_pref_id = ".aqinnesc($cpr_id);
			
		if(sizeof($to_still) > 0) $delete_old_q .= " AND field_id NOT IN (".implode(", ", aqinnesc_array($to_still)).")";
			
		$delete_old = $db->query($delete_old_q);
		
		if($db->error){
				
			$return_data['status'] = 'failed';
			$return_data['error'] = '28379289';
			$return_data['line'] = __LINE__;
			$return_data['technical']['e'] = $db->error;
			
			return json_encode($return_data);
			
		}
		
		// Actualitzem les preferencies simples
		$update_prefs_q = "UPDATE card_preferences 
			SET main_color = ".aqinnesc($preferences->mainColor).",
				name_highlight = ".($preferences->nameHighlight ? 1 : 0).", 
				qr_inverted = ".($preferences->qrInverted ? 1 : 0).", 
				print_centered = ".($preferences->printCentered ? 1 : 0).", 
				bleed_marks = ".($preferences->bleedMarks ? 1 : 0)." 
			WHERE user_id = ".aqinnesc($user_id);
			
		$update_prefs = $db->query($update_prefs_q);
		
		if($db->error){
				
			$return_data['status'] = 'failed';
			$return_data['error'] = '8374983738';
			$return_data['line'] = __LINE__;
			$return_data['technical']['e'] = $db->error;
			
			return json_encode($return_data);
			
		}
		
		$return_data['status'] = 'success';
		
		return json_encode($return_data);
		
	}
	
	function _AddDefaultCardPreferences($user_id){
		
		global $db;
		
		$return_data = initialize_return_data();
		
		$fixed_fields = get_fixed_fields();
		$now = get_now();
		
		$insert_default_q = "INSERT INTO card_preferences 
				(user_id, main_color, name_highlight, qr_inverted, print_centered, bleed_marks, date_created, date_modified)
			VALUES 
				(".aqinnesc($user_id).", 'darkblue', 1, 1, 1, 1, ".aqinnesc($now).", ".aqinnesc($now).")";
				
		$insert_default = $db->query($insert_default_q);
		
		if($db->error){
			
			$return_data['status'] = 'failed';
			$return_data['error'] = '826837874';
			$return_data['line'] = __LINE__;
			$return_data['technical']['e'] = $db->error;
			
			return json_encode($return_data);
			
		}
		
		$return_data['status'] = 'success';
		$return_data['data']['preferences']['id'] = $db->insert_id;
		
		return json_encode($return_data);
		
	}
	
?>