<?php

	function _GetUserProperties($user_id){
		
		global $db;
		
		$return_data = initialize_return_data();
		
		$get_properties_q = "SELECT 
				upr.id AS id, 
				upr.name AS name, 
				upr.value AS value, 
				upr.is_fixed AS is_fixed,
				IF(cfi.id IS NULL, 0, cfi.position) AS card_position
			FROM user_properties upr
			LEFT JOIN card_preferences cpr 
				ON cpr.user_id = ".aqinnesc($user_id)." 
			LEFT JOIN card_fields cfi 
				ON cfi.card_pref_id = cpr.id 
					AND cfi.field_id = upr.id 
			WHERE upr.user_id = ".aqinnesc($user_id)." 
				AND upr.date_deleted IS NULL 
			ORDER BY upr.is_fixed DESC, upr.name ASC";
				
		$get_properties = $db->query($get_properties_q);
		
		if($db->error){
			
			$return_data['status'] = 'failed';
			$return_data['error'] = '82372987922';
			$return_data['line'] = __LINE__;
			$return_data['technical']['e'] = $db->error;
			
			return json_encode($return_data);
			
		}else{
			
			$return_data['status'] = 'success';
			$return_data['data']['properties'] = array();
			
			while($prop = $get_properties->fetch_object()){
				
				$p = array();
				$p['id'] = $prop->id;
				$p['name'] = $prop->name;
				$p['value'] = strip_tags($prop->value);
				$p['isFixed'] = $prop->is_fixed == '1';
				$p['cardPosition'] = $prop->card_position;
				
				$return_data['data']['properties'][] = $p;
				
			}
			
		}
		
		return json_encode($return_data);
		
	}
	
	function _SaveUserProperties($user_id, $properties){
		
		global $db;
		
		$return_data = initialize_return_data();
		$now = get_now();
		
		$to_add = array();
		$to_update = array();
		$ids_still = array();
		
		foreach($properties as $property){
			
			if(property_is_defined($property->id) && $property->id[0] != '-'){
				
				// Senyal que existeix a la bbdd, s'ha d'actualitzar
				$to_update[] = $property;
				// I a més, sabem que encara està, per tant no s'ha d'esborrar
				$ids_still[] = $property->id;
				
			}else{
				
				$to_add[] = $property;
				
			}
		
		}
		
		if(sizeof($ids_still) > 0){
				
			$delete_old_q = "UPDATE user_properties 
				SET date_deleted = ".aqinnesc($now)." 
				WHERE user_id = ".aqinnesc($user_id)." 
					AND id NOT IN (".implode(",", aqinnesc_array($ids_still)).") 
					AND is_fixed != 1";
					
			$delete_old = $db->query($delete_old_q);
			
			if($db->error){
					
				$return_data['status'] = 'failed';
				$return_data['error'] = '28379289';
				$return_data['line'] = __LINE__;
				$return_data['technical']['e'] = $db->error;
				
				return json_encode($return_data);
				
			}
		
		}
		
		// Com és un Update no podem fer una multiquery: preparem un prepared-statement
		$stmt = $db->prepare("UPDATE user_properties 
				SET name = ?, 
					value = ? 
				WHERE id = ?");
		
		$stmt->bind_param("ssi", $prop_name, $prop_value, $prop_id);
		
		foreach($to_update as $prop){
			
			$prop_id = $prop->id;
			$prop_name = $prop->name;
			$prop_value = strip_tags($prop->value);
			$stmt->execute();
			
			if($db->error){
				
				$return_data['status'] = 'failed';
				$return_data['error'] = '2883625241';
				$return_data['line'] = __LINE__;
				$return_data['technical']['e'] = $db->error;
				
				return json_encode($return_data);
				
			}
			
		}
		
		// Ara procedim a afegir les noves propietats
		$to_add_qs = array();
		
		foreach($to_add as $prop){
			
			$to_add_qs[] = "(".aqinnesc($user_id).", ".aqinnesc($prop->name).", ".aqinnesc(strip_tags($prop->value)).", 0, ".aqinnesc($now).")";
			
		}
		
		if(sizeof($to_add_qs) > 0){
			
			$add_q = "INSERT INTO user_properties 
					(user_id, name, value, is_fixed, date_created) 
				VALUES " . implode(", ", $to_add_qs);
			
			$add = $db->query($add_q);
			
			if($db->error){
				
				$return_data['status'] = 'failed';
				$return_data['error'] = '8279289329736';
				$return_data['line'] = __LINE__;
				$return_data['technical']['e'] = $db->error;
				
				return json_encode($return_data);
				
			}
			
		}
		
		$return_data['status'] = 'success';
		
		return json_encode($return_data);
		
	}
	
	
	function _AddFixedFields($user_id){
		
		global $db;
		
		$return_data = initialize_return_data();
		
		$fixed_fields = get_fixed_fields();
		$now = get_now();
		$qs = array();
		
		foreach($fixed_fields as $field){
			
			$qs[] = "(".aqinnesc($user_id).", ".aqinnesc($field->name).", '', 1, ".aqinnesc($now).")";
			
		}
		
		$add_q = "INSERT INTO user_properties 
				(user_id, name, value, is_fixed, date_created)
			VALUES ".implode(", ", $qs);
		
		$add = $db->query($add_q);
		
		if($db->error || !$add){
			
			$return_data['status'] = 'failed';
			$return_data['error'] = '376489393';
			$return_data['line'] = __LINE__;
			$return_data['technical']['e'] = $db->error;
			
			return json_encode($return_data);
			
		}else{
			
			$return_data['status'] = 'success';
			
		}
		
		return json_encode($return_data);
		
	}
	
?>